/* -*- C++ -*- Time-stamp: <21 Nov 00 19:00:51 Michael Bischoff> */

/*****************************************************************************/
/*									     */
/*									     */
/*	QMatchIt version 1.0 -- file field.h				     */
/*									     */
/*	Include file for the field base class of QMatchIt		     */
/*	written by Michael Bischoff (Michael.Bischoff@gmx.net)		     */
/*	November-2000							     */
/*	see COPYRIGHT.qmatchit for Copyright details			     */
/*									     */
/*									     */
/*****************************************************************************/

#ifndef GAME_H
#define GAME_H

class field;

class game {
    field ***rows;
    field *engaged_with;
    field *cursor;
    int cursor_x;
    int cursor_y;

    int nextpoints;		// score for next block
    int score;			// total score for this level
    char *lv_title;
    char *lv_desc;

    // next three: level logic is done inside the mainwindow logic
    // const char *lv_filename;
    // bool advanceLevel;
    // int level;

    void deleteTexts();			// free memory for ascii strings
    void disengageCursor();		// free object associated with cursor
    void engageCursor(field *fp);	// associate object with cursor (grab)
    void transitionToEmpty(int x, int y);
    void moveBlock(int x, int y, int x2, int y2);
    void changeAllMagnets();
    void checkForDestruction(int x, int y, int dx, int dy);
    void destruct(field *fp);
    bool heldByMagnet(int x, int y);
    void setRows(const char **dat);

public:
    game();
    ~game();
    enum key_dir { Key_Up = 0, Key_Down = 1, Key_Left = 2, Key_Right = 3, Key_Select = 4 };
    void gameKey(enum key_dir);		// keyPress in game
    void editKey(int ascii);		// keyPress in edit mode
    bool levelIsSolved();
    int getScore()		const { return score; }
    const char *getLvTitle()	const { return lv_title; }
    const char *getLvDesc()	const { return lv_desc; }
    void clear();
    bool loadFromFile(const char *filename, int level);
    void load(const char *);
    void save(const char *);
    void animate();
    static void advance_all();
};

#endif
